/**
 * 战斗
 */
// --------------------------导入-------------------------
import { Tr } from 'pi_ucore/db/mgr';
import { Env } from 'pi_ucore/db/env';
import { asyncSpawnReceive } from 'pi_pt/util/pid';
import { sendMsg } from 'pi_pt/util/pid_rpc';
import { read } from "pi_common_tools/util/db_tool";

import { getShengqiInfo } from 'game_fight_entry/base/entry';
import { handleFightRsEventRpc } from "game_divine_zone_server/server/fight_result_handle.event";

import { PidFight, FightList, pidState } from "game_fight_entry/base/pid_fight.struct";
import { PidfightNums } from "game_common/xls/config_style/config_style.struct";

declare var env: Env;
declare var ESProcess: any;

/**
 * @desc 对外方法，初始化战斗虚拟机
 */
export const createFightPids = async () => {
    await env.dbMgr.write(async (tr: Tr) => {
        let pids = new Map();
        const pidsCount = PidfightNums || 16;
        for (let i = 0; i < pidsCount; i++) {
            let pid = await asyncSpawnReceive("");
            let _pidState = new pidState();
            _pidState.key = pid;
            _pidState.state = "pending";
            _pidState.pidType = i == 0 ? PidType.regular : PidType.normal;
            pids.set(pid, _pidState);
        }
        let _PidFight = new PidFight();
        _PidFight.key = "pid_fight";
        _PidFight.pids = pids;
        await tr.modify([{ ware: "memory", tab: PidFight._$info.name, key: "pid_fight", value: _PidFight }], 1000, true);
    });
}

const getFreePids = async (tr, isNormal: boolean = false) => {
    let _PidFight = await read(tr, PidFight, "pid_fight") as PidFight;
    if (!_PidFight) return new Map();
    let pids = new Map();
    for (let value of _PidFight.pids) {
        let normal = isNormal ? value[1].pidType == PidType.normal : true;
        if (value[1].state == "pending" && normal) {
            pids.set(value[0], value[1]);
        }
    }
    return pids;
}

/**
 * 对外API，让外部调用虚拟机战斗，接受参数为一组虚拟机战斗数据
 */
export const receivePidFightList = async (tr: Tr, pid: number, lists: FightList[]) => {
    const pids = await getFreePids(tr, true);
    if (pids.size == 0) return;
    const _pids = Array.from(pids);
    // 根据开的虚拟机，分布执行这些战斗
    // let sendPromiseList = [];
    let num = 0;
    for(let value of lists) {
        const fightPid = Number(_pids[num][0]);
        // 获得当前阵容的圣器信息
        let shengqiInfo;
        if (tr && value.fKey && value.teamNo) {
            const ownShengqi: any = value.rid ? (await getShengqiInfo(tr, value.fKey, value.rid, value.teamNo || 1)) : null;
            const enemyShengqi: any = value.otherRid ? (await getShengqiInfo(tr, value.fKey, value.otherRid, value.teamNo || 1)) : null;
            shengqiInfo = {own: ownShengqi, enemy: enemyShengqi};
        }
        // sendPromiseList.push(sendMsg(fightPid, 'process_send_msg', {...value, shengqiInfo}))
        sendMsg(fightPid, 'process_send_msg', {...value, shengqiInfo}).then(async (result) => {
            await handleFightRsEventRpc(result);
        }).catch(async (err) => {
            await handleFightRsEventRpc({
                key: value.key,
                r: 4,
            });
            console.log(`receivePidFightList err fightKey: ${value.key}, err: ${err}!!!`)
        });
        num++;
        if (!_pids[num + 1]) num = 0;
    }
    // Promise.all(sendPromiseList).then(async (result) => {
    //     for(let value of result) {
    //         await sendMsg(pid, value.mqttNotifyName, value);
    //     }
    // })
}

// 固定的pid
export const sampleFight = async (tr: Tr, data: FightList, pid?: number) => {
    if (!pid) {
        let freePids = await getFreePids(tr);
        if (freePids.size == 0) return undefined;
        for (let value of freePids) {
            if (value[1].state == "pending" && value[1].pidType == PidType.regular) {
                pid = value[0];
                break;
            }
        }
    }
    if (!pid) {
        return undefined;
    }
    let _PidFight = await read(tr, PidFight, "pid_fight") as PidFight;
    if (_PidFight.pids.size == 0) {
        return undefined;
    }
    if (!_PidFight.pids.has(pid)) {
        return undefined;
    }
    if (_PidFight.pids.get(pid).state !== "pending") {
        return undefined;
    }
    let result = await sendMsg(Number(pid), 'process_send_msg', data) as Promise<FightResult>;
    return result;
}

enum PidType {
    normal = "normal", // 非固定类型的虚拟机，用于定时服务的虚拟机使用
    regular = "regular" // 固定类型的虚拟机，不用于定时服务
}

interface FightResult {
    r: number, // 1: 胜利 2: 失败 0: 平局(失败)
    recount: Array<number[]>, // 战报 [伤害、承伤、治疗]
    play_list: Array<number>, // 己方阵容下标，这个下标是战斗中战斗对象ID
    monster_list: Array<number>, // 敌方阵容下标，这个下标是战斗中战斗对象ID
    fList: Array<any[]>, // 己方所有伙伴的头像，伙伴id，伙伴等级，伙伴星级等
    mList: Array<any[]>, // 敌方所有伙伴的头像，伙伴id，伙伴等级，伙伴星级等
    count: number, // 战斗回合数
    seed: number, // 战斗随机种子
    ownDdzl: number, // 己方大道之力
    enemyDdzl: number, // 敌方大道之力
    fightVersion: string, // 战斗版本号
    hpInfo, // 剩余血量信息
    deadList, // 死亡列表
    ownAttr: any, // 己方属性，主要用于战斗回放
    enemyAttr: any, // 敌方属性，主要用于战斗回放
    shengqiInfo: any, // 圣器信息
}