import { getPower } from "game_power_get/base/monster";
import { clientRpc } from 'pi_network/client_net/net_env';
import { getHerosZhenfaAttrClient } from "game_power_get/rpc/power_check.event";

import { getCfg } from "pi_common_tools/util/db_tool";
import { MonsterBase } from "game_monster_base/xls/monster/monster.struct";

export const checkDefendIsMonster = (defend): 'pve' | 'pvp' => {
    for(let value of defend) {
        if (!value) continue;
        return (value.base_info.type && value.base_info.type == "monster") ? 'pve' : 'pvp';
    }
    return 'pve';
}

/**获取编队界面返回的攻守双方的属性 */
export const getFightNeedAttr = async (attackHeros: Map<number, any>, defendHeos: Map<number, any>) => {
    let ownAttr: Map<number, any> = new Map();
    let enemyAttr: Map<number, any> = new Map();
    if (attackHeros && attackHeros.size > 0) {
        for(let value of attackHeros) {
            let res = await getOneTeamAttr(value)
            ownAttr.set(Number(value[0]), res);
        }
    }
    if (defendHeos && defendHeos.size > 0) {
        for(let value of defendHeos) {
            let type = checkType(value[1]);
            if (type == "monster") {
                let arr: number[][] = [];
                value[1].forEach(v => arr.push([v.heroTid, v.level]))
                enemyAttr.set(Number(value[0]), getPower(arr))
            }else {
                let res = await getOneTeamAttr(value)
                enemyAttr.set(Number(value[0]), res);
            }
        }
    }
    return {
        ownAttr,
        enemyAttr
    }
}

export const getOneTeamAttr = async (heros: Map<number, any> | any[]) => {
    let arr: number[] = [];
    let value = Array.isArray(heros) ? heros : Array.from((new Map()).set(1, heros))[0];
    value[1].forEach((v) => {
        if (!v) return;
        if (typeof(v) == "number") {
            arr.push(v);
        }else {
            v.heroId && arr.push(v.heroId)
        }
    });
    let result = await clientRpc(getHerosZhenfaAttrClient._$topic, arr);
    let res = [];
    for(let vv of value[1]) {
        if (Array.isArray(vv)) {
            if (vv && vv[1]) {
                let rIndex = result.findIndex(v => vv[1] == v.base_info.baseId);    
                result[rIndex].base_info.site = vv[0];
                res[vv[0] - 1] = result[rIndex];
            }else {
                res[vv[0] - 1] = 0;
            }
        }else {
            if(vv.heroId) {
                let rIndex = result.findIndex(v => vv.heroId == v.base_info.baseId);    
                result[rIndex].base_info.site = vv.site;
                res[vv.site - 1] = result[rIndex];
            }else {
                res[vv.site - 1] = 0;
            }
        }
    }
    return res;
}

const checkType = (arg) => {
    let index = arg.findIndex(v => v && v.heroTid);
    let id = arg[index].heroTid;
    if (getCfg(MonsterBase, id)) {
        return 'monster'
    }else {
        return "partner"
    }
}

/**
 * 获取兽海战斗信息
 * @param monster boss轮换配置表的具体行
 * @param cfgPath 兽海掉落奖励配置表
 * @returns 
 */
export const getMosnterSeaInfo = (monster, cfgPath?: string) => {
    let enemyAttr = [];
    let monsterAttr = getPower([monster.monsterInfo]);
    for(let key in monster.seat) {
        let value = monster.seat[key];
        if(!value || value == -1) {
            enemyAttr.push(0);
        }else {
            monsterAttr[0].base_info.index = value;
            enemyAttr.push({...monsterAttr[0]});
        }
    }
    return {
        enemyAttr,
        fightBg: monster.scene,
        monsterSeaHurtCfg: cfgPath,
        bossId: monster.monsterInfo[0]
    }
}